require 'socket'
require 'ipaddr'

class UDPMulticast
  def initialize(multicast_addr = '225.4.5.6', port = 553, ttl = 1)
    @multicast_addr = multicast_addr
    @port = port
    @ttl = ttl
    @server = nil
    @client = nil
    client_socket
  end

  def server
    @server
  end

  def ttl
    @ttl
  end

  def port
    @port
  end

  def multicast_addr
    @multicast_addr
  end

  def client
    @client
  end

  def run!
    create_socket
    socket_opt @server, Socket::IP_ADD_MEMBERSHIP, multicast_member
    socket_opt @server, Socket::IP_MULTICAST_TTL, @ttl
    socket_opt @server, Socket::IP_TTL, @ttl
    bind
  end

  def client_socket
    open_socket
    socket_opt @client, Socket::IP_MULTICAST_TTL, @ttl
    socket_opt @client, Socket::IP_TTL, @ttl
  end

  private

  def multicast_member
    IPAddr.new(@multicast_addr).hton + IPAddr.new('0.0.0.0').hton
  end

  def create_socket
    @server = UDPSocket.new
  end

  def open_socket
    @client = UDPSocket.open
  end

  def socket_opt(socket, opt, val)
    socket.setsockopt Socket::IPPROTO_IP, opt, val
  end

  def bind
    @server.bind Socket::INADDR_ANY, @port
  end
end
